from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *
from Products.ImageRepository.config import *

from Products.ATContentTypes.atct import ATFolderSchema
from Products.ATContentTypes.atct import ATCTBTreeFolder
from Products.ATContentTypes.interfaces import IATFolder
from Products.ATContentTypes.content.base import updateActions

from Products.ImageRepository.interfaces import IImageRepository
from Products.ImageRepository import wrappers

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import ModifyPortalContent

# Zope 3 imports
from zope.interface import implements

import urllib
from ZTUtils.Zope import complex_marshal


ImageRepositorySchema = ATFolderSchema.copy()

class ImageRepository(ATCTBTreeFolder):
    """ImageRepository content type.
    """

    implements(IImageRepository)

    content_icon   = 'imagerepository_icon.gif'
    meta_type      = 'ImageRepository'
    archetype_name = 'Image Repository'
    immediate_view = 'imagerepository_view'
    default_view   = 'imagerepository_view'
    suppl_views    = ()
    typeDescription= ''
    assocMimetypes = ()
    assocFileExt   = ()
    cmf_edit_kws   = ()

    schema = ImageRepositorySchema

    security = ClassSecurityInfo()

    actions = updateActions(ATCTBTreeFolder,
        ({
            'id'          : 'edit_keywords',
            'name'        : 'Tags',
            'action'      : 'string:${object_url}/imagerepository_tagging',
            'permissions' : (ModifyPortalContent,),
         },
        )
    )

    __implements__ = ATCTBTreeFolder.__implements__, IATFolder

    security.declarePrivate('at_post_create_script')
    def at_post_create_script(self):
        # disallow the adding of any type
        self.setConstrainTypesMode(1)
        self.setLocallyAllowedTypes(['Image'])
        self.setImmediatelyAddableTypes(['Image'])

    security.declareProtected('View', 'getUniqueKeywordsFromResults')
    def getUniqueKeywordsFromResults(self, results):
        keywords = {}
        for item in results:
            subjects = item.Subject
            if subjects is not None:
                for keyword in subjects:
                    keywords[keyword] = keywords.get(keyword, 0) + 1
        return keywords

    security.declareProtected('View', 'getSearchKeywordsFromResults')
    def getSearchKeywordsFromResults(self, results):
        keywords = self.getUniqueKeywordsFromResults(results)
        results_len = len(results)
        for key, count in keywords.items():
            if count == results_len:
                del keywords[key]
        return keywords

    security.declareProtected('View', 'getImageRepositoryPath')
    def getImageRepositoryPath(self):
        return "/".join(self.getPhysicalPath())

    security.declareProtected('View', 'getImageRepositoryURL')
    def getImageRepositoryURL(self):
        return self.absolute_url()

    security.declareProtected('View', 'queryImageRepository')
    def queryImageRepository(self, query=None, REQUEST=None):
        if REQUEST is None:
            request = self.REQUEST
        else:
            request = REQUEST
        portal_catalog = getToolByName(self, 'portal_catalog')
        if query is None:
            query = {}
        query['portal_type'] = 'Image'
        query['path'] = self.getImageRepositoryPath()
        keywords = request.get('keywords', None)
        if keywords is not None:
            query['Subject'] = {'query':keywords, 'operator':'and'}
        return portal_catalog(query)

    security.declareProtected('View', 'makeImageRepositoryQuery')
    def makeImageRepositoryQuery(self, data=None, add=None, omit=None):
        d = {}
        if data is not None:
            d.update(data)
        if add is not None:
            for key in add:
                if not d.has_key(key):
                    d[key] = add[key]
                else:
                    if isinstance(d[key], list):
                        d[key] = d[key] + add[key]
        if omit is not None:
            for key in omit:
                if d.has_key(key):
                    value = omit[key]
                    if not isinstance(value, list):
                        value = [value]
                    for v in value:
                        d[key] = [x for x in d[key] if x!=v]
        uq = urllib.quote
        qlist = complex_marshal(d.items())
        for i in range(len(qlist)):
            k, m, v = qlist[i]
            qlist[i] = '%s%s=%s' % (uq(k), m, uq(str(v)))
    
        return '&'.join(qlist)

registerType(ImageRepository, PROJECTNAME)
