# Zope3 imports
from zope.interface import Interface


class IImageRepository(Interface):
    """An image repository.
    """

    def getKeywords():
        """Return a sequence of keywords for this repository.
        """


class IKeyword(Interface):
    """A keyword, with an optionally associated image, that knows
    how to find content tagged with it.
    """

    def getId():
        """Return the keyword/id for this topic.
        """

    def getTitle():
        """Return title.
        """

    def title_or_id():
        """Return the title, if not empty, else the id.
        """

    def getDescription():
        """Return the description for this topic.
        """

    def getLazyEntries():
        """Return all those entries that are tagged with this topic/keyword
        and that are in the published state.
        """

    def getLastModified():
        """Return a DateTime instance for when a post in this ITopic was
        last modified.
        """

    def __len__():
        """Return the number of posts for this ITopic.
        """

    def __str__():
        """Return a string representation of one or more the keywords.
        """


